#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
from pytest import raises, approx


def test():
    import {{package_name}}
    pass


if __name__ == "__main__":
    import os

    basename = os.path.basename(__file__)
    pytest.main([basename, "-s", "--tb=native"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This script provides dev-ops tools for python project development.
"""

from __future__ import print_function
from pathlib_mate import Path
from setup import package


def fixcode(**kwargs):
    """
    auto pep8 format all python file in ``source code`` and ``tests`` dir.
    """
    # repository direcotry
    repo_dir = Path(__file__).parent.absolute()

    # source code directory
    source_dir = Path(repo_dir, package.__name__)

    if source_dir.exists():
        print("Source code locate at: '%s'." % source_dir)
        print("Auto pep8 all python file ...")
        source_dir.autopep8(**kwargs)
    else:
        print("Source code directory not found!")

    # unittest code directory
    unittest_dir = Path(repo_dir, "tests")
    if unittest_dir.exists():
        print("Unittest code locate at: '%s'." % unittest_dir)
        print("Auto pep8 all python file ...")
        unittest_dir.autopep8(**kwargs)
    else:
        print("Unittest code directory not found!")

    print("Complete!")


if __name__ == "__main__":
    fixcode()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Integrate pyenv, tox, travis-ci.

pyenv is a great software to management multiple Python version and switch
between them.

It assumes you are developing your library on one version, and test over few
other version.

This module helps to convert pyenv specified version into tox, travis-ci
compatible version.
"""

import re
from collections import OrderedDict

import bs4
import requests

python_build_url = "https://github.com/pyenv/pyenv/tree/master/plugins/python-build/share/python-build"


def _get_pyenv_versions_from_github():
    """
    Get all available pyenv version.
    """
    html = requests.get(python_build_url).text
    try:
        soup = bs4.BeautifulSoup(html, "html.parser")
    except:  # pragma: no cover
        soup = bs4.BeautifulSoup(html)

    versions = list()
    for div in soup.find_all("div", class_="file-wrap"):
        for table in div.find_all("table"):
            for td in table.find_all("td", class_="content"):
                for a in td.find_all("a"):
                    try:
                        title = a["title"]
                        if title != "patches":
                            if title == a.text:
                                versions.append(title)
                    except:  # pragma: no cover
                        pass

    return versions


def get_pyenv_versions():
    try:
        return _get_pyenv_versions_from_github()
    except:  # pragma: no cover
        return list()


pyenv_versions = get_pyenv_versions()


def validate_pyenv_ver(pyenv_ver):
    if pyenv_versions:
        if pyenv_ver not in pyenv_versions:
            raise ValueError(
                "%s is not a valid pyenv supported python version! "
                "Go to %s see supported version list." % (
                    pyenv_ver, python_build_url)
            )


def pyenv_ver_to_tox_ver(pyenv_ver):
    """
    Convert pyenv supported version to tox supported version.

    - `pyenv supported version <https://github.com/pyenv/pyenv/tree/master/plugins/python-build/share/python-build>`_.
    - `tox supoorted version <http://tox.readthedocs.io/en/latest/example/basic.html#a-simple-tox-ini-default-environments>`_.

    Example:

        2.7.13 -> py27
        pypy3-2.4.0 -> pypy3
    """
    validate_pyenv_ver(pyenv_ver)

    special_version_mapper = OrderedDict([
        ("jython", "jython"),
        ("pypy3", "pypy3"),
        ("pypy", "pypy"),
        ("anaconda3", "py3"),
        ("anaconda2", "py2"),
        ("anaconda", "py"),
    ])

    for ver in special_version_mapper:
        if ver in pyenv_ver:
            return special_version_mapper[ver]

    # regular python version like 2/3.x.x
    py_ver_regex = "\d+.\d+"
    try:
        py_ver = re.findall(py_ver_regex, pyenv_ver)[0]

        if float(py_ver) < 2.6:  # 2.5 or earlier
            return "py2"
        if py_ver.startswith("2.6"):
            return "py26"
        if py_ver.startswith("2.7"):
            return "py27"

        if float(py_ver) < 3.3:  # 3.2 or earlier
            return "py3"
        if py_ver.startswith("3.3"):
            return "py33"
        if py_ver.startswith("3.4"):
            return "py34"
        if py_ver.startswith("3.5"):
            return "py35"
        if py_ver.startswith("3.6"):
            return "py36"
        if py_ver.startswith("3.7"): # pragma: no cover
            return "py37"

        # for future version
        return "py" + pyenv_ver.replace(".", "")  # pragma: no cover
    except:  # pragma: no cover
        raise ValueError(
            "Cannot find supported tox version for %s!" % pyenv_ver)


def pyenv_ver_to_travis_ver(pyenv_ver):
    """
    Convert pyenv supported version to travis supported version.

    - `pyenv supported version <https://github.com/pyenv/pyenv/tree/master/plugins/python-build/share/python-build>`_.
    - `travis.ci supoorted version <https://docs.travis-ci.com/user/languages/python/>`_.

    Example:

        2.7.13 -> 2.7
        pypy3-2.4.0 -> pypy3
    """
    validate_pyenv_ver(pyenv_ver)

    # solve pypy, anaconda
    special_version_mapper = OrderedDict([
        ("pypy3", "pypy3"),
        ("pypy", "pypy"),
        ("anaconda3", "3.4"),
        ("anaconda2", "2.7"),
        ("anaconda", "2.7"),
    ])

    for ver in special_version_mapper:
        if ver in pyenv_ver:
            return special_version_mapper[ver]

    # not supported version
    not_supported_version = [
        "ironpython", "jython", "miniconda", "pyston", "stackless",
    ]
    for ver in not_supported_version:
        if ver in pyenv_ver:
            raise ValueError("%s are not supported by travis!" % pyenv_ver)

    # dev version
    if pyenv_ver.endswith("-dev"):
        return pyenv_ver

    # regular python version like 2/3.x.x
    py_ver_regex = "\d+.\d+"

    try:
        py_ver = re.findall(py_ver_regex, pyenv_ver)[0]

        if float(py_ver) < 2.6:  # 2.5 or earlier
            raise ValueError("%s is not supported by travis!" % py_ver)

        if py_ver.startswith("2.6"):
            return "2.6"
        if py_ver.startswith("2.7"):
            return "2.7"

        if float(py_ver) < 3.2:  # 3.1 or earlier
            raise ValueError("%s is not supported by travis!" % py_ver)

        if py_ver.startswith("3.3"):
            return "3.3"
        if py_ver.startswith("3.4"):
            return "3.4"
        if py_ver.startswith("3.5"):
            return "3.5"
        if py_ver.startswith("3.6"):
            return "3.6"
        if py_ver.startswith("3.7"): # pragma: no cover
            return "3.7"

        return py_ver
    except:
        raise ValueError(
            "Cannot find supported tox version for %s!" % pyenv_ver)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This script can generate automate scripts for open source python project.
"""

from __future__ import print_function
import sys
import datetime
from os import walk, mkdir, getcwd

import click
from jinja2 import Template
from pathlib_mate.pathlib import Path

try:
    from . import integrate
    from .util import read, write
    from .version import __version__
except:  # pragma: no cover
    from pygitrepo import integrate
    from pygitrepo.util import read, write
    from pygitrepo.version import __version__


py_ver_major = sys.version_info.major
py_ver_minor = sys.version_info.minor
py_ver_micro = sys.version_info.micro

py_ver_long = "%s.%s.%s" % (py_ver_major, py_ver_minor, py_ver_micro)
py_ver_short = "%s.%s" % (py_ver_major, py_ver_minor)


def simple_render(template_content, kwargs):  # pragma: no cover
    """
    Rend a jinja2 styled template.

    :param template_content: str
    :param kwargs: dict
    """
    temp_prefix = "{{ PyGitRepo_Temp_Prefix____%s }}"
    mapper1 = dict()
    """
    {"{{ name }}": "{{ PyGitRepo_Temp_Prefix____name }}"
    """
    mapper2 = dict()
    """
    {"{{ PyGitRepo_Temp_Prefix____name }}": value}
    """
    for i, (key, value) in enumerate(kwargs.items()):
        temp_key = temp_prefix % i
        mapper1["{{ %s }}" % key] = temp_key
        mapper2[temp_key] = str(value)
    for old, new in mapper1.items():
        template_content = template_content.replace(old, new)
    for old, new in mapper2.items():
        template_content = template_content.replace(old, new)
    return template_content


def jinja2_render(template_content, kwargs):  # pragma: no cover
    template = Template(template_content)
    return template.render(**kwargs)


def is_none_or_empty(value):
    """
    Test if a value is None or 'False' things.
    """
    if (value is None) or (bool(value) is False):
        return True
    else:
        return False

SUPPORT_DOC_HOST_SERVICE = ["none", "rtd", "s3"]

def initiate_project(package_name=None,
                     repo_name=None,
                     github_username=None,
                     supported_py_ver=None,
                     author_name=None,
                     author_email=None,
                     maintainer_name=None,
                     maintainer_email=None,
                     license="MIT",
                     rtd_name=None,
                     s3_bucket=None,
                     doc_service="none",
                     verbose=True,
                     **kwargs):
    """
    Generate skeleton of project files.

    :param package_name: your python package name, it should be able to install
        via ``pip install <package_name>``, and will be published on
        https://pypi.python.org/pypi/<package_name>

    :param repo_name: github repository name, the github link will be:
        https://github.com//<github_username>/<repo_name>

    :param github_username: github username, the github link will be:
        https://github.com//<github_username>/<repo_name>

    :param supported_py_ver: list of Python version you want to support,
        it has to be a valid pyenv version.
        available Python Version names can be found at:
        https://github.com/pyenv/pyenv/blob/master/plugins/python-build/share/python-build.

    :param author_name: author name
    :param author_email: author email
    :param maintainer_name: maintainer name
    :param maintainer_email: maintainer email

    :param license: currently only support MIT license, you can manually replace
        the license file yourself.

    :param rtd_name: Read the doc project name, doc will be host at:
        http://<rtd_name>.readthedocs.io/

    :param s3_bucket: AWS S3 bucket name, doc will be host at
        http://<s3_bucket>.s3.amazonaws.com/<package_name>/index.html. You need to
        config your bucket to allow all public get traffic. Tutorial is here:
        http://docs.aws.amazon.com/AmazonS3/latest/dev/WebsiteHosting.html

    :param doc_service: str, doc service you want to use, one of
        "none", "rtd", "s3".

    :param verbose: bool, toggle on / off the log info.
    """
    if is_none_or_empty(package_name):  # pragma: no cover
        raise ValueError("`package_name` can't be None!")

    if is_none_or_empty(repo_name):  # pragma: no cover
        repo_name = "%s-project" % package_name

    if is_none_or_empty(github_username):  # pragma: no cover
        raise ValueError("`github_username` can't be None!")
    else:
        repo_url = "https://github.com/{github_username}/{repo_name}" \
            .format(github_username=github_username, repo_name=repo_name)

    # if not provided, use the current intepreter.
    if is_none_or_empty(supported_py_ver):  # pragma: no cover
        supported_py_ver = [py_ver_long, ]

    supported_py_ver_for_tox = set()
    for py_ver in supported_py_ver:
        try:
            supported_py_ver_for_tox.add(
                integrate.pyenv_ver_to_tox_ver(py_ver)
            )
        except Exception as e:  # pragma: no cover
            print(e)
    supported_py_ver_for_tox = list(supported_py_ver_for_tox)
    supported_py_ver_for_tox.sort()

    supported_py_ver_for_tarvis = set()
    for py_ver in supported_py_ver:
        try:
            supported_py_ver_for_tarvis.add(
                integrate.pyenv_ver_to_travis_ver(py_ver)
            )
        except Exception as e:  # pragma: no cover
            print(e)
    supported_py_ver_for_tarvis = list(supported_py_ver_for_tarvis)
    supported_py_ver_for_tarvis.sort()

    if license != "MIT":  # pragma: no cover
        license = "MIT"
    click.secho(
        ("Initiate with a MIT license, "
         "you can manually change the file yourself!"),
        fg="red",
    )

    if is_none_or_empty(rtd_name) is False:  # pragma: no cover
        doc_domain_rtd = "https://{rtd_name}.readthedocs.io" \
            .format(rtd_name=rtd_name)
    else: # pragma: no cover
        rtd_name = "Unknown_ReadTheDocs_Project_Name"

    if is_none_or_empty(s3_bucket) is False:  # pragma: no cover
        doc_domain_s3 = "http://{s3_bucket}.s3.amazonaws.com/{package_name}" \
            .format(s3_bucket=s3_bucket, package_name=package_name)
    else: # pragma: no cover
        s3_bucket = "Unknwon_S3_Bucket_Name"

    doc_domain = None
    if doc_service == "none": # pragma: no cover
        click.secho(
            ("You don't have a website to host your documents! "
             "You could use either https://readthedocs.org or AWS S3."),
            fg="green"
        )
    elif doc_service == "rtd": # pragma: no cover
        doc_domain = doc_domain_rtd
    elif doc_service == "s3": # pragma: no cover
        doc_domain = doc_domain_s3
    else: # pragma: no cover
        raise ValueError("doesn't recognize doc host service '%s'!" % doc_service)

    click.secho(
        (
        "There's author introduction file at {repo_name}/docs/source/author.rst, "
        "you should change it to your own introduction.") \
            .format(repo_name=repo_name),
        fg="red",
    )

    if not maintainer_name:  # pragma: no cover
        maintainer_name = author_name

    if not maintainer_email:  # pragma: no cover
        maintainer_email = author_email

    kwargs_ = dict(
        pygitrepo_version=__version__,
        package_name=package_name,
        repo_name=repo_name,
        github_username=github_username,
        repo_url=repo_url,

        supported_py_ver=supported_py_ver,
        supported_py_ver_for_tox=supported_py_ver_for_tox,
        supported_py_ver_for_travis=supported_py_ver_for_tarvis,

        py_ver_major=py_ver_major,
        py_ver_minor=py_ver_minor,
        py_ver_micro=py_ver_micro,

        year=datetime.datetime.utcnow().year,
        today=datetime.date.today(),

        license=license,

        author_name=author_name,
        author_email=author_email,
        maintainer_name=maintainer_name,
        maintainer_email=maintainer_email,

        rtd_name=rtd_name,
        s3_bucket=s3_bucket,
        doc_domain=doc_domain,
    )
    kwargs_.update(kwargs)

    # Create files
    print("Initate '%s' from template ..." % repo_name)

    template_dir = Path(__file__).absolute().change(
        new_basename="{{ repo_name }}")
    output_dir = Path(getcwd(), repo_name).absolute()

    file_count = 0
    for src_dir, dir_list, file_list in walk(template_dir.abspath):
        if src_dir.endswith("__pycache__"):  # pragma: no cover
            continue

        # find destination directory
        dst_dir = Path(
            jinja2_render(
                src_dir.replace(template_dir.abspath, output_dir.abspath, 1),
                kwargs_,
            )
        )

        # make destination directory
        try:
            if verbose:
                print("    Create '%s' ..." % dst_dir)
            mkdir(dst_dir.abspath)
        except:  # pragma: no cover
            pass

        # files
        for filename in file_list:
            if filename.endswith(".pyc") or filename.endswith(
                    ".DS_Store"):  # pragma: no cover
                continue

            src = Path(src_dir, filename)
            dst = Path(
                jinja2_render(
                    Path(dst_dir.abspath, filename).abspath,
                    kwargs_,
                )
            )

            try:  # text file
                content = jinja2_render(read(src.abspath), kwargs_)
                if verbose:
                    print("    Create '%s' ..." % dst)
                write(content, dst.abspath)
            except:  # binary file
                src.copyto(new_abspath=dst.abspath, overwrite=True)

            file_count += 1

    print("    COMPLETE! %s files created." % file_count)


@click.command()  # pragma: no cover
@click.option(
    "--package_name",
    prompt="(REQUIRED) Your Package Name (e.g. pip)",
)
@click.option(
    "--repo_name",
    prompt="(optional) Your Repository Name (e.g. pip-project)",
    default="",
)
@click.option(
    "--github_username",
    prompt="(REQUIRED) Your Github Username",
)
@click.option(
    "--supported_py_ver",
    prompt=(
        "(optional) Enter python version list your package will support, "
        "seperate by comma. "
        "For example: '2.7.13, 3.4.6'. "
        "All available version are listed here: "
        "https://github.com/pyenv/pyenv/tree/master/plugins/python-build/share/python-build"
    ),
    default="",
)
@click.option(
    "--author_name",
    prompt="(optional) Author Name",
    default="",
)
@click.option(
    "--author_email",
    prompt="(optional) Author Email",
    default="",
)
@click.option(
    "--maintainer_name",
    prompt="(optional) Maintainer Name",
    default="",
)
@click.option(
    "--maintainer_email",
    prompt="(optional) Maintainer Email",
    default="",
)
@click.option(
    "--rtd_name",
    prompt=(
          "(optional) If use ReadTheDocs to host your document, "
          "please specify the project name "
          "(will be your domain prefix)."
    ),
    default="",
)
@click.option(
    "--s3_bucket",
    prompt=(
          "(optional) If use AWS S3 to host your document, "
          "please specify the bucket name."
    ),
    default="",
)
@click.option(
    "--doc_service",
    prompt="(REQUIRED) Choose your doc host serivce from %s." % (
        " | ".join(SUPPORT_DOC_HOST_SERVICE), ),
    type=click.Choice(SUPPORT_DOC_HOST_SERVICE),
)
def _initiate_project(package_name,
                      repo_name,
                      github_username,
                      supported_py_ver,
                      author_name,
                      author_email,
                      maintainer_name,
                      maintainer_email,
                      rtd_name,
                      s3_bucket,
                      doc_service):
    if supported_py_ver is None:
        pass
    elif supported_py_ver.strip():
        supported_py_ver = [
            ver.strip().lower() for ver in supported_py_ver.split(",") if
            ver.strip()
        ]
    else:  # pragma: no cover
        supported_py_ver = None

    initiate_project(
        package_name=package_name,
        repo_name=repo_name,
        github_username=github_username,
        supported_py_ver=supported_py_ver,
        author_name=author_name,
        author_email=author_email,
        maintainer_name=maintainer_name,
        maintainer_email=maintainer_email,
        rtd_name=rtd_name,
        s3_bucket=s3_bucket,
        doc_service=doc_service,
    )


if __name__ == "__main__":
    initiate_project()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


def read(path, encoding="utf-8"):
    """Read string from text file.
    """
    with open(path, "rb") as f:
        return f.read().decode(encoding)


def write(s, path, encoding="utf-8"):
    """Write string to text file.
    """
    with open(path, "wb") as f:
        f.write(s.encode(encoding))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
1. Creates auto-generate doc for each module / class / method / variable.
2. Creates Table of Content for sub chapter if you follow this
`Style Guide <http://www.wbh-doc.com.s3.amazonaws.com/docfly/02-sphinx-doc-style-guide/index.html>`_
"""

import docfly
from pathlib_mate import Path
import {{package_name}} as package

source_dir = Path(__file__).absolute().parent.append_parts("source").abspath

# --- Manually Made Doc ---
# Comment this if you don't follow this style guide.
# http://www.wbh-doc.com.s3.amazonaws.com/docfly/02-sphinx-doc-style-guide/index.html
doc = docfly.DocTree(source_dir)
doc.fly(table_of_content_header="Table of Content")

# --- Api Reference Doc ---
package_name = package.__name__
doc = docfly.ApiReferenceDoc(
    package_name,
    dst=source_dir,
    ignored_package=[
        "%s.pkg" % package_name,
        "%s.zzz_ezinstall.py" % package_name,
    ]
)
doc.fly()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Package Description.
"""

__version__ = "0.0.1"
__short_description__ = "Package short description."
__license__ = "{{ license }}"
{% if author_name -%}
__author__ = "{{ author_name }}"
{% endif -%}
{% if author_email -%}
__author_email__ = "{{ author_email }}"
{% endif -%}
{% if maintainer_name -%}
__maintainer__ = "{{ maintainer_name }}"
{% endif -%}
{% if maintainer_email -%}
__maintainer_email__ = "{{ maintainer_email }}"
{% endif -%}
__github_username__ = "{{ github_username }}"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .version import __version__

__short_description__ = (
    "Allow dummies develop Python project like a Pro - "
    "Quickly initiate a python project from scratch."
)
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__maintainer__ = "Sanhe Hu"
__maintainer_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"

try:
    from .cli import initiate_project as init
except ImportError:  # pragma: no cover
    pass
